@@ -17,26 +17,22 @@
  *	along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
  *	Authors:
- *		Marius Tomaschewski <mt@suse.de>
- *		Nirmoy Das <ndas@suse.de>
+ *		Marius Tomaschewski
+ *		Nirmoy Das
  */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include <stdio.h>
-#include <string.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <getopt.h>
-#include <net/if.h>
-#include <net/if_arp.h>
-
-#include <wicked/types.h>
 #include <wicked/netinfo.h>
 #include <wicked/ethtool.h>
 #include <wicked/util.h>
 
+#include <unistd.h>
+#include <getopt.h>
+
+#include <net/if_arp.h>
+
 struct ethtool_args {
 	int	argc;
 	char **	argv;
@@ -138,6 +134,28 @@ cleanup:
 }
 
 
+/*
+ * permanent-address
+ */
+static int
+get_ethtool_perm_hwaddr(const ni_netdev_ref_t *ref, ni_ethtool_t *ethtool, struct ethtool_args *args)
+{
+	ni_hwaddr_t hwaddr;
+	const char *string;
+
+	(void)args;
+	ni_link_address_init(&hwaddr);
+	if (ni_ethtool_get_permanent_address(ref, ethtool, &hwaddr) < 0)
+		return -1;
+
+	if (!(string = ni_link_address_print(&hwaddr)))
+		return -1;
+
+	printf("permanent-address: %s\n", string);
+	return 0;
+}
+
+
 /*
  * link-detected
  */
@@ -1105,6 +1123,9 @@ static const struct ethtool_opt	ethtool_opts[] = {
 	{	"--get-driver-info",	.func	= get_ethtool_driver_info,			},
 	{	"--get-private-flags",	.func	= get_ethtool_priv_flags,			},
 	{	"--get-priv-flags",	.func	= get_ethtool_priv_flags,	.alias = TRUE	},
+	{	"--get-permanent-address",
+					.func	= get_ethtool_perm_hwaddr,			},
+	{	"--get-perm-address",	.func	= get_ethtool_perm_hwaddr,	.alias = TRUE	},
 	{	"--get-link-detected",	.func	= get_ethtool_link_detected,			},
 	{	"--get-link-settings",	.func	= get_ethtool_link_settings,			},
 	{	"--get-wake-on-lan",	.func	= get_ethtool_wake_on_lan,			},
@@ -1121,6 +1142,9 @@ static const struct ethtool_opt	ethtool_opts[] = {
 	{	"--show-driver-info",	.func	= get_ethtool_driver_info,	.alias = TRUE	},
 	{	"--show-private-flags",	.func	= get_ethtool_priv_flags,	.alias = TRUE	},
 	{	"--show-priv-flags",	.func	= get_ethtool_priv_flags,	.alias = TRUE	},
+	{	"--show-permanent-address",
+					.func	= get_ethtool_perm_hwaddr,	.alias = TRUE	},
+	{	"--show-perm-address",	.func	= get_ethtool_perm_hwaddr,	.alias = TRUE	},
 	{	"--show-link-settings",	.func	= get_ethtool_link_settings,	.alias = TRUE	},
 	{	"--show-wake-on-lan",	.func	= get_ethtool_wake_on_lan,	.alias = TRUE	},
 	{	"--show-wol",		.func	= get_ethtool_wake_on_lan,	.alias = TRUE	},
@@ -1206,7 +1230,7 @@ ni_do_ethtool(const char *caller, int argc, char **argv)
 	};
 	int c, n, status = NI_WICKED_RC_USAGE;
 	const struct ethtool_opt *opt;
-	ni_netdev_ref_t ref = { 0, NULL };
+	ni_netdev_ref_t ref = NI_NETDEV_REF_INIT;
 	ni_ethtool_t *ethtool = NULL;
 
 	optind = 1;
@@ -1243,7 +1267,7 @@ ni_do_ethtool(const char *caller, int argc, char **argv)
 	}
 
 	status = NI_WICKED_RC_ERROR;
-	ni_netdev_ref_init(&ref, argv[optind], if_nametoindex(argv[optind]));
+	ni_netdev_ref_set(&ref, argv[optind], ni_netdev_name_to_index(argv[optind]));
 	if (!ref.index) {
 		fprintf(stderr, "%s: cannot find interface with name '%s'\n", argv[0], argv[optind]);
 		goto cleanup;
