@@ -45,6 +45,7 @@
 #include <wicked/bonding.h>
 #include <wicked/bridge.h>
 #include <wicked/vlan.h>
+#include <wicked/ipvlan.h>
 #include <wicked/fsm.h>
 #include <wicked/wireless.h>
 
@@ -452,6 +453,18 @@ ni_ifstatus_show_iftype(const ni_netdev_t *dev, ni_bool_t verbose)
 		}
 		break;
 
+	case NI_IFTYPE_IPVLAN:
+	case NI_IFTYPE_IPVTAP:
+		if (dev->ipvlan) {
+			ni_stringbuf_t tmp = NI_STRINGBUF_INIT_DYNAMIC;
+
+			printf(" %s, mode %s %s", dev->link.lowerdev.name,
+				ni_ipvlan_mode_to_name(dev->ipvlan->mode),
+				ni_ipvlan_format_flags(dev->ipvlan->flags, &tmp) ?: "bridge");
+			ni_stringbuf_destroy(&tmp);
+		}
+		break;
+
 	case NI_IFTYPE_BOND:
 		if (dev->bonding) {
 			printf(", mode %s",
@@ -972,7 +985,7 @@ ni_ifstatus_shutdown_result(ni_fsm_t *fsm, ni_string_array_t *names, ni_ifworker
 		if (!w->kickstarted)
 			continue;
 
-		if (marked && ni_ifworker_array_index(marked, w) < 0)
+		if (marked && ni_ifworker_array_index(marked, w) == -1U)
 			continue;
 
 		if (names && names->count != 0 &&
@@ -1022,7 +1035,7 @@ ni_ifstatus_display_result(ni_fsm_t *fsm, ni_string_array_t *names, ni_ifworker_
 		if (!w || ni_string_empty(w->name))
 			continue;
 
-		if (marked && ni_ifworker_array_index(marked, w) < 0)
+		if (marked && ni_ifworker_array_index(marked, w) == -1U)
 			continue;
 
 		if (names && names->count != 0 &&
