@@ -33,6 +33,9 @@
 #include <limits.h>
 #include <errno.h>
 #include <net/if_arp.h>
+#ifdef HAVE_SYSTEMD_SD_DAEMON_H
+#include <systemd/sd-daemon.h>
+#endif
 
 #include <wicked/types.h>
 #include <wicked/logging.h>
@@ -216,7 +219,7 @@ main(int argc, char **argv)
 		case OPT_TEST_MODE:
 			if (!tester || ni_string_empty(optarg))
 				goto usage;
-			if (ni_dhcp6_mode_name_to_type(optarg, &tester->mode) < 0)
+			if (!ni_dhcp6_mode_parse(&tester->mode, optarg))
 				goto usage;
 			break;
 
@@ -458,8 +461,14 @@ dhcp6_supplicant(void)
 	if (opt_recover_state)
 		dhcp6_recover_state(opt_state_file);
 
+#ifdef HAVE_SYSTEMD_SD_DAEMON_H
+	if (opt_systemd) {
+		sd_notify(0, "READY=1");
+	}
+#endif
+
 	while (!ni_caught_terminal_signal()) {
-		long timeout;
+		ni_timeout_t timeout;
 
 		do {
 			timeout = ni_timer_next_timeout();
