@@ -86,6 +86,7 @@ struct ni_netdev {
 	ni_vlan_t *		vlan;
 	ni_vxlan_t *		vxlan;
 	ni_macvlan_t *		macvlan;
+	ni_ipvlan_t *		ipvlan;
 	ni_wireless_t *		wireless;
 	ni_openvpn_t *		openvpn;
 	ni_tuntap_t *		tuntap;
@@ -220,6 +221,8 @@ extern ni_team_t *	ni_netdev_get_team(ni_netdev_t *);
 extern ni_vlan_t *	ni_netdev_get_vlan(ni_netdev_t *);
 extern ni_vxlan_t *	ni_netdev_get_vxlan(ni_netdev_t *);
 extern ni_macvlan_t *	ni_netdev_get_macvlan(ni_netdev_t *);
+extern ni_ipvlan_t *	ni_netdev_get_ipvlan(ni_netdev_t *);
+extern ni_ipvlan_t *	ni_netdev_get_ipvtap(ni_netdev_t *);
 extern ni_bridge_t *	ni_netdev_get_bridge(ni_netdev_t *);
 extern ni_ovs_bridge_t *ni_netdev_get_ovs_bridge(ni_netdev_t *);
 extern ni_wireless_t *	ni_netdev_get_wireless(ni_netdev_t *);
@@ -235,6 +238,7 @@ extern void		ni_netdev_set_bonding(ni_netdev_t *, ni_bonding_t *);
 extern void		ni_netdev_set_team(ni_netdev_t *, ni_team_t *);
 extern void		ni_netdev_set_vlan(ni_netdev_t *, ni_vlan_t *);
 extern void		ni_netdev_set_vxlan(ni_netdev_t *, ni_vxlan_t *);
+extern void		ni_netdev_set_ipvlan(ni_netdev_t *, ni_ipvlan_t *);
 extern void		ni_netdev_set_macvlan(ni_netdev_t *, ni_macvlan_t *);
 extern void		ni_netdev_set_bridge(ni_netdev_t *, ni_bridge_t *);
 extern void		ni_netdev_set_ovs_bridge(ni_netdev_t *, ni_ovs_bridge_t *);
