@@ -83,6 +83,8 @@ extern unsigned int		ni_objectmodel_service_registry_count(void);
 #define NI_OBJECTMODEL_DUMMY_INTERFACE		NI_OBJECTMODEL_INTERFACE ".Dummy"
 #define NI_OBJECTMODEL_VLAN_INTERFACE		NI_OBJECTMODEL_INTERFACE ".VLAN"
 #define NI_OBJECTMODEL_VXLAN_INTERFACE		NI_OBJECTMODEL_INTERFACE ".VXLAN"
+#define NI_OBJECTMODEL_IPVLAN_INTERFACE		NI_OBJECTMODEL_INTERFACE ".IPVLAN"
+#define NI_OBJECTMODEL_IPVTAP_INTERFACE		NI_OBJECTMODEL_INTERFACE ".IPVTAP"
 #define NI_OBJECTMODEL_TEAM_INTERFACE		NI_OBJECTMODEL_INTERFACE ".Team"
 #define NI_OBJECTMODEL_BONDING_INTERFACE	NI_OBJECTMODEL_INTERFACE ".Bond"
 #define NI_OBJECTMODEL_TUN_INTERFACE		NI_OBJECTMODEL_INTERFACE ".TUN"
