@@ -170,7 +170,7 @@ ni_factory_device_up(ni_fsm_t *fsm, ni_ifworker_t *w)
 	ifmarker.target_range.max = NI_FSM_STATE_MAX;
 	ifmarker.persistent = w->control.persistent;
 
-	ni_ifworker_array_append(&ifmarked, w);
+	ni_ifworker_array_append_ref(&ifmarked, w);
 	ni_fsm_mark_matching_workers(fsm, &ifmarked, &ifmarker);
 	ni_ifworker_array_destroy(&ifmarked);
 
@@ -378,7 +378,7 @@ ni_managed_device_up(ni_managed_device_t *mdev, const char *origin)
 	ifmarker.target_range.max = NI_FSM_STATE_MAX;
 	ifmarker.persistent = w->control.persistent;
 
-	ni_ifworker_array_append(&ifmarked, w);
+	ni_ifworker_array_append_ref(&ifmarked, w);
 
 	/* Binding: this validates the XML configuration document,
 	 * resolves any references to other devices (if there are any),
