@@ -1,14 +1,39 @@
 /*
- * Handling of network and link layer addresses.
- * Currently, just about formatting addresses for display.
+ *	Network and link layer addresses handling.
  *
- * Copyright (C) 2009-2012 Olaf Kirch <okir@suse.de>
- * Copyright (C) 2012-2013 Marius Tomaschewski <mt@suse.de>
+ *	Copyright (C) 2009-2012 Olaf Kirch <okir@suse.de>
+ *	Copyright (C) 2012-2023 SUSE LLC
+ *
+ *	This program is free software; you can redistribute it and/or modify
+ *	it under the terms of the GNU General Public License as published by
+ *	the Free Software Foundation; either version 2 of the License, or
+ *	(at your option) any later version.
+ *
+ *	This program is distributed in the hope that it will be useful,
+ *	but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *	GNU General Public License for more details.
+ *
+ *	You should have received a copy of the GNU General Public License
+ *	along with this program. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *	Authors:
+ *		Olaf Kirch
+ *		Marius Tomaschewski
  */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#include <wicked/logging.h>
+#include <wicked/netinfo.h>
+#include <wicked/time.h>
+#include <wicked/route.h>
+
+#include "refcount_priv.h"
+#include "slist_priv.h"
+#include "util_priv.h"
+
 #include <string.h>
 #include <stdlib.h>
 #include <limits.h>
@@ -22,11 +47,6 @@
 #include <linux/if_infiniband.h>
 #include <netlink/netlink.h>
 
-#include <wicked/logging.h>
-#include <wicked/netinfo.h>
-#include <wicked/socket.h>
-#include <wicked/route.h>
-#include "util_priv.h"
 
 #define	NI_ADDRESS_ARRAY_CHUNK		16
 
@@ -40,29 +60,40 @@ static const unsigned char *__ni_sockaddr_data(const ni_sockaddr_t *, unsigned i
 /*
  * ni_address functions
  */
-static ni_address_t *
-do_address_new(void)
+static inline ni_bool_t
+ni_address_init(ni_address_t *ap)
 {
-	ni_address_t *ap;
-
-	ap = xcalloc(1, sizeof(*ap));
 	if (ap) {
-		ap->refcount = 1;
+		memset(ap, 0, sizeof(*ap));
 		ap->cache_info.valid_lft = NI_LIFETIME_INFINITE;
 		ap->cache_info.preferred_lft = NI_LIFETIME_INFINITE;
+		return TRUE;
 	}
-	return ap;
+	return FALSE;
 }
 
+static inline void
+ni_address_destroy(ni_address_t *ap)
+{
+	ni_string_free(&ap->label);
+}
+
+extern ni_define_refcounted_new(ni_address);
+extern ni_define_refcounted_ref(ni_address);
+extern ni_define_refcounted_hold(ni_address);
+extern ni_define_refcounted_free(ni_address);
+extern ni_define_refcounted_drop(ni_address);
+extern ni_define_refcounted_move(ni_address);
+
 ni_address_t *
-ni_address_new(int af, unsigned int prefix_len, const ni_sockaddr_t *local_addr, ni_address_t **list_head)
+ni_address_create(int af, unsigned int prefix_len, const ni_sockaddr_t *local_addr, ni_address_t **list_head)
 {
 	ni_address_t *ap;
 
 	if (local_addr && local_addr->ss_family != af)
 		return NULL;
 
-	ap = do_address_new();
+	ap = ni_address_new();
 	if (!ap)
 		return NULL;
 
@@ -72,21 +103,10 @@ ni_address_new(int af, unsigned int prefix_len, const ni_sockaddr_t *local_addr,
 	if (local_addr)
 		ap->local_addr = *local_addr;
 
-	if (list_head) {
+	if (list_head)
 		ni_address_list_append(list_head, ap);
-	}
-	return ap;
-}
 
-ni_address_t *
-ni_address_ref(ni_address_t *ap)
-{
-	if (ap) {
-		ni_assert(ap->refcount);
-		ap->refcount++;
-		return ap;
-	}
-	return NULL;
+	return ap;
 }
 
 ni_bool_t
@@ -114,7 +134,7 @@ ni_address_clone(const ni_address_t *src)
 {
 	ni_address_t *dst;
 
-	dst = do_address_new();
+	dst = ni_address_new();
 	if (ni_address_copy(dst, src))
 		return dst;
 
@@ -122,20 +142,6 @@ ni_address_clone(const ni_address_t *src)
 	return NULL;
 }
 
-void
-ni_address_free(ni_address_t *ap)
-{
-	if (ap) {
-		ni_assert(ap->refcount);
-		ap->refcount--;
-		if (ap->refcount != 0)
-			return;
-
-		ni_string_free(&ap->label);
-		free(ap);
-	}
-}
-
 ni_bool_t
 ni_address_equal_ref(const ni_address_t *ap1, const ni_address_t *ap2)
 {
@@ -319,6 +325,18 @@ ni_address_is_mngtmpaddr(const ni_address_t *laddr)
 	return laddr->flags & IFA_F_MANAGETEMPADDR ? TRUE : FALSE;
 }
 
+ni_bool_t
+ni_address_is_nodad(const ni_address_t *laddr)
+{
+	return laddr->flags & IFA_F_NODAD;
+}
+
+ni_bool_t
+ni_address_is_noprefixroute(const ni_address_t *laddr)
+{
+	return laddr->flags & IFA_F_NOPREFIXROUTE;
+}
+
 ni_bool_t
 ni_address_can_reach(const ni_address_t *laddr, const ni_sockaddr_t *gw)
 {
@@ -365,6 +383,24 @@ ni_address_set_mngtmpaddr(ni_address_t *laddr, ni_bool_t temporary)
 		laddr->flags &= ~IFA_F_MANAGETEMPADDR;
 }
 
+void
+ni_address_set_nodad(ni_address_t *laddr, ni_bool_t nodad)
+{
+	if (nodad)
+		laddr->flags |= IFA_F_NODAD;
+	else
+		laddr->flags &= ~IFA_F_NODAD;
+}
+
+void
+ni_address_set_noprefixroute(ni_address_t *laddr, ni_bool_t noprefixroute)
+{
+	if (noprefixroute)
+		laddr->flags |= IFA_F_NOPREFIXROUTE;
+	else
+		laddr->flags &= ~IFA_F_NOPREFIXROUTE;
+}
+
 unsigned int
 ni_address_valid_lft(const ni_address_t *ap, const struct timeval *current)
 {
@@ -395,13 +431,12 @@ ni_address_lft_is_preferred(const ni_address_t *ap, const struct timeval *curren
 /*
  * ni_address list functions
  */
-void
-ni_address_list_append(ni_address_t **list, ni_address_t *ap)
-{
-	while (*list)
-		list = &(*list)->next;
-	*list = ap;
-}
+extern ni_define_slist_append(ni_address);
+extern ni_define_slist_remove(ni_address);
+extern ni_define_slist_delete(ni_address);
+extern ni_define_slist_destroy(ni_address);
+extern ni_define_slist_copy(ni_address);
+extern ni_define_slist_count(ni_address);
 
 void
 ni_address_list_dedup(ni_address_t **list)
@@ -426,55 +461,18 @@ ni_address_list_dedup(ni_address_t **list)
 	}
 }
 
-unsigned int
-ni_address_list_count(ni_address_t *list)
-{
-	unsigned int count = 0;
-	ni_address_t *ap;
-
-	for (ap = list; ap != NULL; ap = ap->next)
-		count++;
-	return count;
-}
-
 ni_address_t *
 ni_address_list_find(ni_address_t *list, const ni_sockaddr_t *addr)
 {
 	ni_address_t *ap;
 
-	for (ap = list; ap != NULL; ap = ap->next) {
+	ni_slist_foreach(list, ap) {
 		if (ni_sockaddr_equal(&ap->local_addr, addr))
 			return ap;
 	}
 	return NULL;
 }
 
-ni_bool_t
-__ni_address_list_remove(ni_address_t **list, ni_address_t *ap)
-{
-	ni_address_t **pos, *cur;
-
-	for (pos = list; (cur = *pos) != NULL; pos = &cur->next) {
-		if (cur == ap) {
-			*pos = cur->next;
-			ni_address_free(cur);
-			return TRUE;
-		}
-	}
-	return FALSE;
-}
-
-void
-ni_address_list_destroy(ni_address_t **list)
-{
-	ni_address_t *ap;
-
-	while ((ap = *list) != NULL) {
-		*list = ap->next;
-		ni_address_free(ap);
-	}
-}
-
 void
 ni_address_array_init(ni_address_array_t *array)
 {
@@ -1432,7 +1430,7 @@ ni_link_address_parse(ni_hwaddr_t *hwa, unsigned short arp_type, const char *str
 	if ((len = ni_parse_hex(string, tmp.data, NI_MAXHWADDRLEN)) < 0)
 		return -1;
 
-	if (ni_link_address_length(arp_type) != (unsigned int)len)
+	if (arp_type != ARPHRD_VOID && ni_link_address_length(arp_type) != (unsigned int)len)
 		return -1;
 
 	memcpy(hwa->data, tmp.data, len);
@@ -1654,7 +1652,7 @@ ni_lifetime_left(unsigned int lifetime, const struct timeval *acquired, const st
 
 	if (timercmp(current, acquired, >)) {
 		timersub(current, acquired, &dif);
-		if (lifetime >= dif.tv_sec) {
+		if ((unsigned long)lifetime >= (unsigned long)dif.tv_sec) {
 			lifetime -= dif.tv_sec;
 			return lifetime;
 		}
@@ -1697,4 +1695,3 @@ ni_address_cache_info_rebase(ni_address_cache_info_t *res, const ni_address_cach
 		res->preferred_lft = NI_LIFETIME_EXPIRED;
 	#undef rebase_lft
 }
-
