@@ -37,6 +37,7 @@
 #include "util_priv.h"
 #include "leasefile.h"
 #include "appconfig.h"
+#include "addrconf.h"
 #include "auto6.h"
 
 #define NI_AUTO6_UPDATER_DELAY		500	/* initial delay timeout in ms         */
@@ -100,6 +101,7 @@ ni_auto6_new(const ni_netdev_t *dev)
 	if (auto6) {
 		auto6->enabled = TRUE;
 		auto6->update = NI_TRISTATE_DEFAULT;
+		ni_netdev_ref_init(&auto6->device);
 		ni_netdev_ref_set(&auto6->device, dev->name, dev->link.ifindex);
 	}
 	return auto6;
@@ -401,12 +403,14 @@ ni_auto6_lease_address_update(ni_netdev_t *dev, ni_addrconf_lease_t *lease, cons
 
 	if ((la = ni_address_list_find(lease->addrs, &ap->local_addr))) {
 		if (ap->owner != NI_ADDRCONF_NONE && ap->owner != NI_ADDRCONF_AUTOCONF) {
+			unsigned int plen = la->prefixlen;
+
 			changed = TRUE;
-			__ni_address_list_remove(&lease->addrs, la);
+			ni_address_list_delete(&lease->addrs, la);
 			ni_debug_verbose(NI_LOG_DEBUG, NI_TRACE_IPV6|NI_TRACE_AUTOIP,
 					"%s: removed address %s/%u in %s:%s lease (owner %s)",
 					dev->name,
-					ni_sockaddr_print(&la->local_addr), la->prefixlen,
+					ni_sockaddr_print(&ap->local_addr), plen,
 					ni_addrfamily_type_to_name(lease->family),
 					ni_addrconf_type_to_name(lease->type),
 					ni_addrconf_type_to_name(ap->owner));
@@ -422,7 +426,7 @@ ni_auto6_lease_address_update(ni_netdev_t *dev, ni_addrconf_lease_t *lease, cons
 					ni_addrconf_type_to_name(ap->owner));
 		}
 	} else
-	if ((la = ni_address_new(ap->family, ap->prefixlen, &ap->local_addr, &lease->addrs))) {
+	if ((la = ni_address_create(ap->family, ap->prefixlen, &ap->local_addr, &lease->addrs))) {
 		changed = TRUE;
 		ni_address_copy(la, ap);
 		ni_debug_verbose(NI_LOG_DEBUG, NI_TRACE_IPV6|NI_TRACE_AUTOIP,
@@ -572,12 +576,14 @@ ni_auto6_on_address_event(ni_netdev_t *dev, ni_event_t event, const ni_address_t
 
 	case NI_EVENT_ADDRESS_DELETE:
 		if ((la = ni_address_list_find(lease->addrs, &ap->local_addr))) {
+			unsigned int plen = la->prefixlen;
+
 			changed = TRUE;
-			__ni_address_list_remove(&lease->addrs, la);
+			ni_address_list_delete(&lease->addrs, la);
 			ni_debug_verbose(NI_LOG_DEBUG, NI_TRACE_IPV6|NI_TRACE_AUTOIP,
 					"%s: deleted address %s/%u in %s:%s lease (owner %s)",
 					dev->name,
-					ni_sockaddr_print(&la->local_addr), la->prefixlen,
+					ni_sockaddr_print(&ap->local_addr), plen,
 					ni_addrfamily_type_to_name(lease->family),
 					ni_addrconf_type_to_name(lease->type),
 					ni_addrconf_type_to_name(ap->owner));
@@ -717,7 +723,7 @@ ni_auto6_on_nduseropt_events(ni_netdev_t *dev, ni_event_t event)
 	ni_addrconf_lease_t *lease;
 	unsigned int lifetime;
 	struct timeval now;
-	ni_bool_t changed;
+	ni_bool_t changed = FALSE;
 
 	if (!dev)
 		return;
@@ -945,8 +951,8 @@ ni_auto6_acquire_run(void *user_data, const ni_timer_t *timer)
 		/*
 		 * the link address was already ready to use
 		 * (e.g. wickedd restarted or ifup ; ifup);
-		 * send router solicit ourselfs to get an RA
-		 * update and (re)apply the lease.
+		 * send router solicit ourselves to get an
+		 * RA update and (re)apply the lease.
 		 */
 		auto6->acquire.send_rs--;
 		if (ni_icmpv6_ra_solicit(&auto6->device, NULL)) {
@@ -987,6 +993,7 @@ ni_auto6_acquire(ni_netdev_t *dev, const ni_auto6_request_t *req)
 	if (!dev || !(auto6 = ni_netdev_get_auto6(dev)) || !req || !req->enabled)
 		return -1;
 
+	auto6->enabled = TRUE;
 	ni_uuid_generate(&auto6->uuid);
 	if (!(lease = ni_auto6_get_lease(dev))) {
 		if ((lease = ni_auto6_new_lease(NI_ADDRCONF_STATE_REQUESTING, &auto6->uuid))) {
