@@ -1356,6 +1356,8 @@ ni_dbus_generic_property_set_string_array(ni_dbus_object_t *obj, const ni_dbus_p
 		return FALSE;
 
 	vptr = __property_data(prop, handle, string_array);
+
+	ni_string_array_destroy(vptr);
 	for (i = 0; i < var->array.len; ++i)
 		ni_string_array_append(vptr, var->string_array_value[i]);
 	return TRUE;
@@ -1375,6 +1377,132 @@ ni_dbus_generic_property_parse_string_array(const ni_dbus_property_t *prop, ni_d
 	return TRUE;
 }
 
+dbus_bool_t
+ni_dbus_generic_property_get_object_path(const ni_dbus_object_t *obj, const ni_dbus_property_t *prop,
+					ni_dbus_variant_t *var, DBusError *error)
+{
+	char **vptr;
+	const void *handle;
+
+	if (!(handle = ni_dbus_generic_property_read_handle(obj, prop, error)))
+		return FALSE;
+
+	vptr = __property_data(prop, handle, string);
+	if (*vptr == NULL) {
+		dbus_set_error(error, NI_DBUS_ERROR_PROPERTY_NOT_PRESENT, "property %s not present", prop->name);
+		return FALSE;
+	}
+	ni_dbus_variant_set_object_path(var, *vptr);
+	return TRUE;
+}
+
+dbus_bool_t
+ni_dbus_generic_property_set_object_path(ni_dbus_object_t *obj, const ni_dbus_property_t *prop,
+					const ni_dbus_variant_t *var, DBusError *error)
+{
+	const char *value;
+	char **vptr;
+	void *handle;
+
+	if (!(handle = ni_dbus_generic_property_write_handle(obj, prop, error)))
+		return FALSE;
+
+	if (!ni_dbus_variant_get_object_path(var, &value))
+		return FALSE;
+
+	vptr = __property_data(prop, handle, string);
+	return ni_string_dup(vptr, value);
+}
+
+dbus_bool_t
+ni_dbus_generic_property_parse_object_path(const ni_dbus_property_t *prop, ni_dbus_variant_t *var, const char *string)
+{
+	return ni_dbus_variant_parse(var, string, prop->signature);
+}
+
+dbus_bool_t
+ni_dbus_generic_property_get_object_path_array(const ni_dbus_object_t *obj, const ni_dbus_property_t *prop,
+					ni_dbus_variant_t *var, DBusError *error)
+{
+	const ni_string_array_t *vptr;
+	const void *handle;
+	unsigned int i;
+
+	if (!(handle = ni_dbus_generic_property_read_handle(obj, prop, error)))
+		return FALSE;
+
+	vptr = __property_data(prop, handle, object_path_array);
+
+	ni_dbus_variant_init_object_path_array(var);
+	for (i = 0; i < vptr->count; ++i) {
+		ni_dbus_variant_append_object_path_array(var, vptr->data[i]);
+	}
+	return TRUE;
+}
+
+dbus_bool_t
+ni_dbus_generic_property_set_object_path_array(ni_dbus_object_t *obj, const ni_dbus_property_t *prop,
+					const ni_dbus_variant_t *var, DBusError *error)
+{
+	ni_string_array_t *vptr;
+	unsigned int i;
+	void *handle;
+
+	if (!(handle = ni_dbus_generic_property_write_handle(obj, prop, error)))
+		return FALSE;
+
+	vptr = __property_data(prop, handle, object_path_array);
+
+	ni_string_array_destroy(vptr);
+	for (i = 0; i < var->array.len; ++i)
+		ni_string_array_append(vptr, var->string_array_value[i]);
+	return TRUE;
+}
+
+dbus_bool_t
+ni_dbus_generic_property_parse_object_path_array(const ni_dbus_property_t *prop, ni_dbus_variant_t *var, const char *string)
+{
+	return FALSE;
+}
+dbus_bool_t
+ni_dbus_generic_property_get_byte_array(const ni_dbus_object_t *obj, const ni_dbus_property_t *prop,
+					ni_dbus_variant_t *var, DBusError *error)
+{
+	ni_byte_array_t *vptr;
+	const void *handle;
+
+	if (!(handle = ni_dbus_generic_property_read_handle(obj, prop, error)))
+		return FALSE;
+
+	vptr = __property_data(prop, handle, byte_array);
+	ni_dbus_variant_set_byte_array(var, vptr->data, vptr->len);
+	return TRUE;
+}
+
+dbus_bool_t
+ni_dbus_generic_property_set_byte_array(ni_dbus_object_t *obj, const ni_dbus_property_t *prop,
+					const ni_dbus_variant_t *var, DBusError *error)
+{
+	ni_byte_array_t *vptr;
+	void *handle;
+
+	if (!(handle = ni_dbus_generic_property_write_handle(obj, prop, error)))
+		return FALSE;
+
+	if (!ni_dbus_variant_is_byte_array(var))
+		return FALSE;
+
+	vptr = __property_data(prop, handle, byte_array);
+	ni_byte_array_destroy(vptr);
+	return ni_byte_array_put(vptr, var->byte_array_value, var->array.len) == var->array.len;
+}
+
+dbus_bool_t
+ni_dbus_generic_property_parse_byte_array(const ni_dbus_property_t *prop, ni_dbus_variant_t *var, const char *string)
+{
+	return FALSE;
+}
+
 /*
  * Build an object path from parent path + name
  */
