@@ -102,6 +102,10 @@ extern ni_dbus_service_t	ni_objectmodel_macvlan_service;
 extern ni_dbus_service_t	ni_objectmodel_macvlan_factory_service;
 extern ni_dbus_service_t	ni_objectmodel_macvtap_service;
 extern ni_dbus_service_t	ni_objectmodel_macvtap_factory_service;
+extern ni_dbus_service_t	ni_objectmodel_ipvlan_service;
+extern ni_dbus_service_t	ni_objectmodel_ipvlan_factory_service;
+extern ni_dbus_service_t	ni_objectmodel_ipvtap_service;
+extern ni_dbus_service_t	ni_objectmodel_ipvtap_factory_service;
 extern ni_dbus_service_t	ni_objectmodel_dummy_service;
 extern ni_dbus_service_t	ni_objectmodel_dummy_factory_service;
 extern ni_dbus_service_t	ni_objectmodel_tun_service;
