@@ -1469,8 +1469,9 @@ ni_dhcp4_decode_csr(ni_buffer_t *bp, ni_route_array_t *routes)
 		if (ni_dhcp4_option_get_sockaddr(bp, &gateway) < 0)
 			return -1;
 
-		rp = ni_route_create(prefix_len, &destination, &gateway, 0, NULL);
-		ni_route_array_append(routes, rp);
+		rp = ni_route_create(prefix_len, &destination, &gateway, 0);
+		ni_route_array_append_ref(routes, rp);
+		ni_route_free(rp);
 	}
 
 	if (bp->underflow)
@@ -1611,8 +1612,9 @@ ni_dhcp4_decode_static_routes(ni_buffer_t *bp, ni_route_array_t *routes)
 		rp = ni_route_create(guess_prefix_len_sockaddr(&destination),
 				&destination,
 				&gateway,
-				0, NULL);
-		ni_route_array_append(routes, rp);
+				0);
+		ni_route_array_append_ref(routes, rp);
+		ni_route_free(rp);
 	}
 
 	if (bp->underflow)
@@ -1644,8 +1646,9 @@ ni_dhcp4_decode_routers(ni_buffer_t *bp, ni_route_array_t *routes)
 		if (!ni_sockaddr_is_specified(&gateway))
 			continue;
 
-		if ((rp = ni_route_create(0, NULL, &gateway, 0, NULL)))
-			ni_route_array_append(routes, rp);
+		rp = ni_route_create(0, NULL, &gateway, 0);
+		ni_route_array_append_ref(routes, rp);
+		ni_route_free(rp);
 	}
 
 	if (bp->underflow)
@@ -1853,7 +1856,7 @@ ni_dhcp4_apply_routes(ni_addrconf_lease_t *lease, ni_route_array_t *routes)
 			continue;
 		if (ni_sockaddr_is_specified(&rp->nh.gateway))
 			continue;
-		ni_route_array_append(&temp, ni_route_ref(rp));
+		ni_route_array_append_ref(&temp, rp);
 	}
 
 	/* now the routes with a gateway - add a
@@ -1870,8 +1873,8 @@ ni_dhcp4_apply_routes(ni_addrconf_lease_t *lease, ni_route_array_t *routes)
 		for (ap = lease->addrs; !added && ap; ap = ap->next) {
 			if (!ni_address_can_reach(ap, &rp->nh.gateway))
 				continue;
-			ni_route_array_append(&temp, ni_route_ref(rp));
-			added = TRUE;
+			if (ni_route_array_append_ref(&temp, rp))
+				added = TRUE;
 		}
 		/* or there is a device route allowing to reach it */
 		for (j = 0; !added && j < temp.count; ++j) {
@@ -1884,16 +1887,20 @@ ni_dhcp4_apply_routes(ni_addrconf_lease_t *lease, ni_route_array_t *routes)
 						&r->destination,
 						&rp->nh.gateway))
 				continue;
-			ni_route_array_append(&temp, ni_route_ref(rp));
-			added = TRUE;
+			if (ni_route_array_append_ref(&temp, rp))
+				added = TRUE;
 		}
 
 		/* otherwise, automatically prepend a device route */
 		if (!added) {
-			unsigned int len = ni_af_address_length(rp->family);
-			r = ni_route_create(len * 8, &rp->nh.gateway, NULL, 0, NULL);
-			ni_route_array_append(&temp, r);
-			ni_route_array_append(&temp, ni_route_ref(rp));
+			unsigned int plen = ni_af_address_prefixlen(rp->family);
+
+			if (plen) {
+				r = ni_route_create(plen, &rp->nh.gateway, NULL, 0);
+				ni_route_array_append_ref(&temp, r);
+				ni_route_free(r);
+			}
+			ni_route_array_append_ref(&temp, rp);
 		}
 	}
 	ni_route_tables_add_routes(&lease->routes, &temp);
