@@ -59,6 +59,8 @@ static const ni_intmap_t	ni_linktype_name_map[] = {
 	{ "bond",		NI_IFTYPE_BOND },
 	{ "vlan",		NI_IFTYPE_VLAN },
 	{ "vxlan",		NI_IFTYPE_VXLAN },
+	{ "ipvlan",		NI_IFTYPE_IPVLAN},
+	{ "ipvtap",		NI_IFTYPE_IPVTAP},
 	{ "macvlan",		NI_IFTYPE_MACVLAN },
 	{ "macvtap",		NI_IFTYPE_MACVTAP },
 	{ "wireless",		NI_IFTYPE_WIRELESS },
@@ -113,6 +115,8 @@ static const ni_intmap_t	ni_linkinfo_kind_map[] = {
 	{ "team",		NI_IFTYPE_TEAM },
 	{ "vlan",		NI_IFTYPE_VLAN },
 	{ "vxlan",		NI_IFTYPE_VXLAN },
+	{ "ipvlan",		NI_IFTYPE_IPVLAN },
+	{ "ipvtap",		NI_IFTYPE_IPVTAP },
 	{ "macvlan",		NI_IFTYPE_MACVLAN },
 	{ "macvtap",		NI_IFTYPE_MACVTAP },
 	{ "tun",		NI_IFTYPE_TUN },
